﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.BinaryVectorEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.ScatterSearch {
  /// <summary>
  /// An operator that combines solutions to three trial solutions.
  /// </summary>
  [Item("SolutionCombinationMethod", "An operator that combines solutions to three trial solutions.")]
  [StorableClass]
  public sealed class SolutionCombinationMethod : SingleSuccessorOperator {
    #region Parameter properties
    private ScopeParameter CurrentScopeParameter {
      get { return (ScopeParameter)Parameters["CurrentScope"]; }
    }
    #endregion

    #region Properties
    public IScope CurrentScope {
      get { return CurrentScopeParameter.ActualValue; }
    }
    #endregion

    [StorableConstructor]
    private SolutionCombinationMethod(bool deserializing) : base(deserializing) { }
    private SolutionCombinationMethod(SolutionCombinationMethod original, Cloner cloner) : base(original, cloner) { }
    public SolutionCombinationMethod() : base() { Initialize(); }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new SolutionCombinationMethod(this, cloner);
    }

    private void Initialize() {
      #region Create parameters
      Parameters.Add(new ScopeParameter("CurrentScope", "The current scope to which the new solutions are added as sub-scopes."));
      #endregion

      #region Create operators
      #endregion

      #region Create operator graph
      #endregion
    }

    public override IOperation Apply() {
      IScope mother = CurrentScope.SubScopes[0];
      IScope father = CurrentScope.SubScopes[1];
      if (mother != null && father != null) {
        BinaryVector motherSolution = mother.Variables["KnapsackSolution"].Value as BinaryVector;
        BinaryVector fatherSolution = father.Variables["KnapsackSolution"].Value as BinaryVector;
        int stepSize = motherSolution.Length / 3;
        IVariable[] children = {
          new Variable("KnapsackSolution", new BinaryVector(motherSolution.Take(stepSize).Concat(motherSolution.Skip(stepSize).Take(stepSize)).Concat(fatherSolution.Skip(2 * stepSize)).ToArray())),
          new Variable("KnapsackSolution", new BinaryVector(motherSolution.Take(stepSize).Concat(fatherSolution.Skip(stepSize).Take(stepSize)).Concat(motherSolution.Skip(2 * stepSize)).ToArray())),
          new Variable("KnapsackSolution", new BinaryVector(fatherSolution.Take(stepSize).Concat(motherSolution.Skip(stepSize).Take(stepSize)).Concat(motherSolution.Skip(2 * stepSize)).ToArray())),
        };
        IScope offspring = new Scope("Offspring");
        foreach (var child in children) {
          IScope solution = new Scope();
          solution.Variables.Add(child);
          offspring.SubScopes.Add(solution);
        }
        CurrentScope.SubScopes.Add(offspring);
      }
      return base.Apply();
    }
  }
}
